#!/bin/bash
# chmod 755 avr2.sh
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/syno/bin:/usr/syno/sbin

# -----------------------------------------------------------------------------------
# Determines the thermal values of your NAS at defined intervals, logs them and
# visualizes them at defined intervals in an SVG graphic.
# -----------------------------------------------------------------------------------
# If you want to stress-test your NAS, open a terminal session for each CPU-Thread and enter:
# -->> cat /dev/zero > /dev/null
# This creates a 100% load for each CPU thread.
# -----------------------------------------------------------------------------------
Version="02.01.11"
Name="avr2"

## --- Initialize a few important values
workingDir="$(dirname "$(readlink -e "$0")")"/""

set -o nounset  # Exit script if try to use an uninitialised variable

# --- Include Libery
. "${workingDir}lib/init.sh"
. "${workingDir}lib/lib.sh"
. "${workingDir}lib/sensors.sh"
. "${workingDir}lib/svg.sh"

# --- Catch Signal: Strg-C / -15
trap lib_STRG_C SIGINT SIGTERM

lib_TimerStart
# ----------------------------------------------------------
# --- Check if an instance of Logger is already running
lib_CheckPid "$Name"
if [ $ExecutionFlag -eq 1 ]; then
    escInv="\033[7m"        # Inverse
    escRst="\033[0m"        # Reset
    echo
    echo -e $escInv"AVR: An instance of AvrLogger is already running."
    echo -e        "AVR: The execution of AvrLogger has been aborted."$escRst
    exit 1
fi
# ----------------------------------------------------------

# --- ToDo: Remove this after development phase (SPK)
DebugFlag=0     # DEBUG = Debug-Modus
# ----------------------------------------------------------
# --- Check if an argument has been passed
# ----------------------------------------------------------
while (( "$#" )); do
    case $1 in
    	DEBUG)
            DebugFlag=1
	    ;;
    esac
    shift 
done

# ----------------------------------------------------------
# --- First, the user home directory must be set up
init_UserDir

if [ $DebugFlag -eq 0 ]; then
    # --- Redirect STDERR to $ERROR (clear $ERROR at Start)
    exec 2>> "$ERROR"
fi

# ----------------------------------------------------------
init_Init               # --- AvrLogger initialize, set default Values & read log.conf
checkSensors            # --- Check availability of Synology Sensors
checkEnv                # --- Check availability of Environmental sensor
SleepTick="0.25"        # --- Set Sleep-Tick-Time to reduce CPU-Usage
init_Log                # --- Init $LOGFILE
init_ListDisk           # --- Init Disks
# ----------------------------------------------------------
svg_Init "10"           # --- Read plot data and prepare
init_PlotData $MaxLines # --- Read log data from previous date
svg_InitPlot            # --- Prepare SVG-Header, Temp-Files
svg_SVG 0               # --- Create SVG-Grafic
# ----------------------------------------------------------
lib_TimerStop
lib_Echo 0 "| AVR: Inizalization completed" "$timerLap |"
lib_Echo 0 "+--------------------------------------------------------------------------------------+"

# ----------------------------------------------------------
# --- Sync Svg-Timer to Time modulo SvgTick
s_time=$(date +%s)
s_time=$[s_time + (SvgTick - (s_time % SvgTick))]

t_time=$[$(date +%s)+$LoopTick]
LoopCount=0
arrOffset=0
l_time=$[$(date +%s)+2]

# ----------------------------------------------------------
# --- Main[endless]loop
# ----------------------------------------------------------
while [ true ]; do

    # ----------------------------------------------------------
	# --- MV-Array: arrOffset / inc. Ring-Buffer
    if [ $arrOffset -eq $arrMax ]
        then arrOffset=1
        else arrOffset=$[arrOffset+1]
    fi

    # --- CPU-Load / ssCpuIdle
    walk=$(lib_getOID "$SynoCom" "$Ip" "1.3.6.1.4.1.2021.11.11.0" "CPU-Load / ssCpuIdle")
    arr[$arrOffset,$TabLoad]=$[(100-walk)*10]

    # ----------------------------------------------------------
    # --- CPU: Form thermal average
	mx=0
    for (( m=1; m<=cpu_count; m++)) do
        read mi < "${Core}temp${cpu_name[$m]}_input"
    	mx=$[mx+mi/100]
    done
	arr[$arrOffset,$TabCpu]=$[mx/(m-1)]
	
    # --- Disk/Rack-Station Thermal-Status
    walk=$(lib_getOID "$SynoCom" "$Ip" "1.3.6.1.4.1.6574.1.2.0" "Thermal-Status")
	arr[$arrOffset,$TabStatus]=$[walk*10]

    # --- --- Environmental sensor
	if [ $LogEnvFlag -ne 0 ]
	    then arr[$arrOffset,$TabEnv]=$(lib_getOID "$LogEnvCom" "$LogEnvIp" "$LogEnvOID" "Environmental sensor")
        else arr[$arrOffset,$TabEnv]=0
    fi

    # ----------------------------------------------------------
    # --- HDD: Get temperatur of all HDD's
    if [ $LoopCount -eq 0 ]; then
        hdd=($(lib_getOID "$SynoCom" "$Ip" "1.3.6.1.4.1.6574.2.1.1.6" "Hard disk temperatures"))
    	mxh=${hdd[0]}; mnh=${hdd[0]}; mvh=0
	    for(( mh=1; mh<${#hdd[@]}; mh++)) do
            # --- Max. temperature
	        if [ $mxh -lt ${hdd[$mh]} ]
	            then mxh=${hdd[$mh]}
		    fi
            # --- Min. temperature
	        if [ $mnh -gt ${hdd[$mh]} ]
	            then mnh=${hdd[$mh]}
		    fi
            # --- Average temperature
	        mvh=$[mvh+${hdd[$mh]}]
	    done
    fi

    arr[$arrOffset,$TabHmax]=$[mxh*10]
    arr[$arrOffset,$TabHmin]=$[mnh*10]
    arr[$arrOffset,$TabHavr]=$[(mvh*10)/(mh-1)]

    # ----------------------------------------------------------
    # --- FAN-Values (PWM-Values) in percent (%)
	if [ $fan_count -gt 0 ]
	then
		mx=0
	   	for (( m=1; m<=fan_count; m++)) do
            read mi < "${i2c}fan${fan_name[$m]}_input"
        	mx=$[mx+mi]
    	done
		mx=$[mx*100/(m-1)/Rpm]
        # --- intercept rounding error ---
		if [ $mx -gt 100 ]
		    then arr[$arrOffset,$TabPwm]=1000
		    else arr[$arrOffset,$TabPwm]=$[mx*10]
	    fi
    else
        arr[$arrOffset,$TabPwm]=0
	fi

    # ----------------------------------------------------------
    # --- i2c Form and average the average temperature
	if [ $i2c_count -gt 0 ]; then
	    mx=0
        for (( m=1; m<=i2c_count; m++)) do
            read mi < "${i2c}temp${i2c_name[$m]}_input"
    	    mx=$[mx+mi/100]
        done
	    arr[$arrOffset,$Tabi2c]=$[mx/(m-1)]
    else
        arr[$arrOffset,$Tabi2c]=0
    fi

    # ----------------------------------------------------------
    # --- Initial Moving Average Filter fill with the first values
	if [ $arr_cnt -eq 0 ]; then
	    arr_cnt=1
        for ((i=2;i<=arrMax;i++)) do
	        for ((j=0;j<=TabLast;j++)) do
                arr[$i,$j]=${arr[1,$j]}
	        done
        done
	fi

    # ----------------------------------------------------------
    # --- Calculate Moving Average Filter
	for ((j=0;j<=TabLast;j++)) do
	    arr[0,$j]=0
    	for ((i=1;i<=arrMax;i++)) do
            arr[0,$j]=$((${arr[0,$j]}+${arr[$i,$j]}))
        done
        arr[0,$j]=$((${arr[0,$j]}/$arr_row))
	done

    # ----------------------------------------------------------
    # --- Output values
    # ----------------------------------------------------------
    LoopCount=$[LoopCount+1]
    if [ $LoopCount -eq $arrDiv ]; then
        lib_AddLine "$LogDat" 1 10080
        lib_AddPlot $MaxLines
        LoopCount=0

        # --- ToDo: Remove this after development phase (SPK)
        if [ $DebugFlag -eq 1 ]; then
            pine=$(date '+%Y.%m.%d %H:%M:%S')
            for ((i=0;i<=TabLast;i++)) do
                pine+=" "${arr[0,$i]}
            done
            lib_Echo 2 "| $pine" "|"
        fi
    fi
    # ----------------------------------------------------------
    # --- END Output
    # ----------------------------------------------------------

    # --- Wait until next AvrTick
	until [ $(date +%s) -ge $t_time ]; do

        # --- Wait until LogTick and Check if Config changed
        if [ $(date +%s) -ge $l_time ]; then
            lib_ConfigTimestamp
            if [ $ConfStatFlag -eq 1 ]; then
                init_ConfigUpdate
                ConfStatFlag=0
            fi
            l_time=$[$(date +%s)+1]
        fi

        # --- Wait until SvgTick
        if [ $(date +%s) -ge $s_time ]; then
            s_time=$[$(date +%s)+$SvgTick]
            svg_SVG $NewLines
            NewLines=0
        fi
        sleep "$SleepTick"
	done

    # --- New AvrTick-Time
    t_time=$[$(date +%s)+$LoopTick]

    # --- Archiving 
    if [ $SvgArchive -gt 0 ]; then
        lib_SvgArchive "$SvgFile" $SvgArchive
    fi

done

# -------- END --------
